@@ -23,6 +23,7 @@ from coralnet_toolbox.Icons import get_icon
 # ----------------------------------------------------------------------------------------------------------------------
 # Classes
 # ----------------------------------------------------------------------------------------------------------------------
+# Note: Field names are case-sensitive and should match the CoralNet export exactly.
 
 
 class ImportViscoreAnnotations(QDialog):
@@ -250,7 +251,7 @@ class ImportViscoreAnnotations(QDialog):
         """Process and import the annotations."""
         try:
             # Map image names to paths
-            image_path_map = {os.path.basename(path): path for path in self.image_window.image_paths}
+            image_path_map = {os.path.basename(path): path for path in self.image_window.raster_manager.image_paths}
 
             # Pre-create required labels
             progress_bar.setWindowTitle("Creating Labels")
@@ -258,17 +259,11 @@ class ImportViscoreAnnotations(QDialog):
 
             for label_code in df['Label'].unique():
                 if pd.notna(label_code):
-                    short_code = long_code = str(label_code)
-                    if not self.label_window.get_label_by_codes(short_code, long_code):
-                        label_id = str(uuid.uuid4())
-                        color = QColor(random.randint(0, 255),
-                                       random.randint(0, 255),
-                                       random.randint(0, 255))
-
-                        self.label_window.add_label_if_not_exists(short_code,
-                                                                  long_code,
-                                                                  color,
-                                                                  label_id)
+                    # Get the label information
+                    short_code = str(label_code)
+                    # Create the label if it does not exist
+                    label = self.label_window.add_label_if_not_exists(short_code)
+                    
                 progress_bar.update_progress()
 
             # Import annotations
