@@ -2,17 +2,17 @@ import warnings
 
 warnings.filterwarnings("ignore", category=DeprecationWarning)
 
-import os
-import json
+import ujson as json
 
 from PyQt5.QtCore import Qt
-from PyQt5.QtWidgets import (QDialog, QFileDialog, QVBoxLayout, QPushButton, QLabel, 
+from PyQt5.QtWidgets import (QDialog, QFileDialog, QVBoxLayout, QPushButton, QLabel,
                              QMessageBox, QGroupBox, QFormLayout, QHBoxLayout, QLineEdit,
                              QApplication)
 
 from coralnet_toolbox.Annotations.QtPatchAnnotation import PatchAnnotation
 from coralnet_toolbox.Annotations.QtPolygonAnnotation import PolygonAnnotation
 from coralnet_toolbox.Annotations.QtRectangleAnnotation import RectangleAnnotation
+from coralnet_toolbox.Annotations.QtMultiPolygonAnnotation import MultiPolygonAnnotation
 
 from coralnet_toolbox.QtProgressBar import ProgressBar
 
@@ -39,8 +39,9 @@ class SaveProject(QDialog):
         self.setup_save_layout()
         # Setup the buttons layout
         self.setup_buttons_layout()
-        
+
     def setup_save_layout(self):
+        """Setup the layout for saving the project."""
         group_box = QGroupBox("Save Project")
         layout = QFormLayout()
 
@@ -58,6 +59,7 @@ class SaveProject(QDialog):
         self.setLayout(main_layout)
 
     def setup_buttons_layout(self):
+        """Setup the layout for the save and cancel buttons."""
         layout = self.layout()
         button_layout = QHBoxLayout()
         self.save_button = QPushButton("Save")
@@ -69,30 +71,32 @@ class SaveProject(QDialog):
         layout.addLayout(button_layout)
 
     def browse_file_path(self):
+        """Open a file dialog to select the save file path."""
         options = QFileDialog.Options()
-        file_path, _ = QFileDialog.getSaveFileName(self, 
-                                                   "Save Project JSON", 
-                                                   "", 
-                                                   "JSON Files (*.json);;All Files (*)", 
+        file_path, _ = QFileDialog.getSaveFileName(self,
+                                                   "Save Project JSON",
+                                                   "",
+                                                   "JSON Files (*.json);;All Files (*)",
                                                    options=options)
         if file_path:
             self.file_path_edit.setText(file_path)
 
     def save_project(self):
+        """Save the project data to a JSON file."""
         file_path = self.file_path_edit.text()
         if file_path:
             self.save_project_data(file_path)
         else:  # Keep it as is
             self.file_path_edit.setText(self.current_project_path)
-        
-    def save_project_data(self, file_path):
 
+    def save_project_data(self, file_path):
+        """Save the project data to a JSON file."""
         # Make cursor busy
         QApplication.setOverrideCursor(Qt.WaitCursor)
 
         try:
             project_data = {
-                'image_paths': self.get_images(),
+                'images': self.get_images(),
                 'labels': self.get_labels(),
                 'annotations': self.get_annotations()
             }
@@ -103,13 +107,13 @@ class SaveProject(QDialog):
             # Update current project path
             self.current_project_path = file_path
 
-            QMessageBox.information(self.annotation_window, 
-                                    "Project Saved", 
+            QMessageBox.information(self.annotation_window,
+                                    "Project Saved",
                                     "Project has been successfully saved.")
 
         except Exception as e:
-            QMessageBox.warning(self.annotation_window, 
-                                "Error Saving Project", 
+            QMessageBox.warning(self.annotation_window,
+                                "Error Saving Project",
                                 f"An error occurred while saving the project: {str(e)}")
 
         finally:
@@ -120,18 +124,31 @@ class SaveProject(QDialog):
         self.accept()
 
     def get_images(self):
+        """Get the list of image objects, including paths, states, and work areas."""
         # Start the progress bar
-        total_images = len(self.image_window.image_paths)
-        progress_bar = ProgressBar(self.label_window, "Exporting Images")
+        total_images = len(self.image_window.raster_manager.image_paths)
+        progress_bar = ProgressBar(self.label_window, "Exporting Image Data")
         progress_bar.show()
         progress_bar.start_progress(total_images)
 
         try:
             export_images = []
-            
+
             # Loop through all of the image paths
-            for image_path in self.image_window.image_paths:
-                export_images.append(image_path)
+            for image_path in self.image_window.raster_manager.image_paths:
+                raster = self.image_window.raster_manager.get_raster(image_path)
+                if raster:
+                    # Get work areas for this raster
+                    work_areas_list = [wa.to_dict() for wa in raster.get_work_areas()]
+
+                    image_data = {
+                        'path': image_path,
+                        'state': {
+                            'checkbox_state': raster.checkbox_state
+                        },
+                        'work_areas': work_areas_list
+                    }
+                    export_images.append(image_data)
                 progress_bar.update_progress()
 
         except Exception as e:
@@ -146,6 +163,7 @@ class SaveProject(QDialog):
         return export_images
 
     def get_labels(self):
+        """Get the list of labels to export."""
         # Start the progress bar
         total_labels = len(self.label_window.labels)
         progress_bar = ProgressBar(self.label_window, "Exporting Labels")
@@ -172,6 +190,7 @@ class SaveProject(QDialog):
         return export_labels
 
     def get_annotations(self):
+        """Get the annotations to export."""
         # Start progress bar
         total_annotations = len(list(self.annotation_window.annotations_dict.values()))
         progress_bar = ProgressBar(self.annotation_window, title="Exporting Annotations")
@@ -203,6 +222,11 @@ class SaveProject(QDialog):
                         'type': 'RectangleAnnotation',
                         **annotation.to_dict()
                     }
+                elif isinstance(annotation, MultiPolygonAnnotation):
+                    annotation_dict = {
+                        'type': 'MultiPolygonAnnotation',
+                        **annotation.to_dict()
+                    }
                 else:
                     raise ValueError(f"Unknown annotation type: {type(annotation)}")
 
@@ -222,6 +246,7 @@ class SaveProject(QDialog):
         return export_annotations
 
     def get_project_path(self):
+        """Get the current project path."""
         return self.current_project_path
 
     def showEvent(self, event):
@@ -235,9 +260,9 @@ class SaveProject(QDialog):
         if self.current_project_path:
             self.file_path_edit.setText(self.current_project_path)
         super().closeEvent(event)
-    
+
     def reject(self):
         """Handle dialog rejection (Cancel or close)"""
         if self.current_project_path:
             self.file_path_edit.setText(self.current_project_path)
-        super().reject()
+        super().reject()
\ No newline at end of file
