@@ -16,9 +16,9 @@ from coralnet_toolbox.Icons import get_icon
 
 class Base(QDialog):
     """
-    Base class for performing batch inference on images for image classification, object detection, 
+    Base class for performing batch inference on images for image classification, object detection,
     and instance segmentation.
-    
+
     :param main_window: MainWindow object
     :param parent: Parent widget
     """
@@ -27,7 +27,7 @@ class Base(QDialog):
         self.main_window = main_window
         self.image_window = main_window.image_window
         self.annotation_window = main_window.annotation_window
-        
+
         self.setWindowIcon(get_icon("coral.png"))
         self.setWindowTitle("Batch Inference")
         self.resize(400, 100)
@@ -48,22 +48,22 @@ class Base(QDialog):
         self.setup_task_specific_layout()
         # Setup the buttons layout
         self.setup_buttons_layout()
-        
+
     def setup_info_layout(self):
         """
         Set up the layout and widgets for the info layout.
         """
         group_box = QGroupBox("Information")
         layout = QVBoxLayout()
-        
+
         # Create a QLabel with explanatory text and hyperlink
         info_label = QLabel("Perform batch inferencing on the selected images. It is highly "
                             "recommended to save the current project before proceeding.")
-        
+
         info_label.setOpenExternalLinks(True)
         info_label.setWordWrap(True)
         layout.addWidget(info_label)
-        
+
         group_box.setLayout(layout)
         self.layout.addWidget(group_box)
 
@@ -78,7 +78,7 @@ class Base(QDialog):
         # Create a button group for the image checkboxes
         image_options_group = QButtonGroup(self)
 
-        self.apply_filtered_checkbox = QCheckBox("Apply to filtered images")
+        self.apply_filtered_checkbox = QCheckBox("▼ Apply to filtered images")
         self.apply_prev_checkbox = QCheckBox("↑ Apply to previous images")
         self.apply_next_checkbox = QCheckBox("↓ Apply to next images")
         self.apply_all_checkbox = QCheckBox("↕ Apply to all images")
@@ -102,7 +102,7 @@ class Base(QDialog):
 
         group_box.setLayout(layout)
         self.layout.addWidget(group_box)
-        
+
     def setup_task_specific_layout(self):
         """
 
@@ -119,7 +119,7 @@ class Base(QDialog):
         button_box.rejected.connect(self.reject)
 
         self.layout.addWidget(button_box)
-    
+
     def update_uncertainty_label(self):
         """
         Update the uncertainty threshold label based on the slider value.
@@ -131,28 +131,43 @@ class Base(QDialog):
     def on_uncertainty_changed(self, value):
         """
         Update the slider and label when the shared data changes.
-        
+
         :param value: New uncertainty threshold value
         """
         self.uncertainty_threshold_slider.setValue(int(value * 100))
         self.uncertainty_threshold_label.setText(f"{value:.2f}")
-        
+
     def get_selected_image_paths(self):
         """
         Get the selected image paths based on the options.
-        
+
         :return: List of selected image paths
         """
+        # Current image path showing
+        current_image_path = self.annotation_window.current_image_path
+        if not current_image_path:
+            return []
+
+        # Determine which images to export annotations for
         if self.apply_filtered_checkbox.isChecked():
-            return self.image_window.filtered_image_paths
+            return self.image_window.table_model.filtered_paths
         elif self.apply_prev_checkbox.isChecked():
-            current_image_index = self.image_window.image_paths.index(self.annotation_window.current_image_path)
-            return self.image_window.image_paths[:current_image_index + 1]
+            if current_image_path in self.image_window.table_model.filtered_paths:
+                current_index = self.image_window.table_model.get_row_for_path(current_image_path)
+                return self.image_window.table_model.filtered_paths[:current_index + 1]
+            else:
+                return [current_image_path]
         elif self.apply_next_checkbox.isChecked():
-            current_image_index = self.image_window.image_paths.index(self.annotation_window.current_image_path)
-            return self.image_window.image_paths[current_image_index:]
+            if current_image_path in self.image_window.table_model.filtered_paths:
+                current_index = self.image_window.table_model.get_row_for_path(current_image_path)
+                return self.image_window.table_model.filtered_paths[current_index:]
+            else:
+                return [current_image_path]
+        elif self.apply_all_checkbox.isChecked():
+            return self.image_window.raster_manager.image_paths
         else:
-            return self.image_window.image_paths
+            # Only apply to the current image
+            return [current_image_path]
 
     def apply(self):
         """
@@ -171,11 +186,11 @@ class Base(QDialog):
         finally:
             # Resume the cursor
             QApplication.restoreOverrideCursor()
-        
+
         self.accept()
 
     def batch_inference(self):
         """
         Perform batch inference on the selected images and annotations.
         """
-        raise NotImplementedError("Subclasses must implement this method.")
\ No newline at end of file
+        raise NotImplementedError("Subclasses must implement this method.")
