@@ -22,7 +22,7 @@ from coralnet_toolbox.QtProgressBar import ProgressBar
 # Classes
 # ----------------------------------------------------------------------------------------------------------------------
 
-
+# TODO Crash might be due to multithreading?
 class Classify(Base):
     def __init__(self, main_window, parent=None):
         super().__init__(main_window, parent)
@@ -146,8 +146,9 @@ class Classify(Base):
             self.image_paths = self.get_selected_image_paths()
             self.preprocess_annotations()
             self.infer_button.setEnabled(True)
-            QMessageBox.information(self, "Preprocessing Complete", 
-                                  f"Successfully preprocessed {len(self.prepared_patches)} annotations.")
+            QMessageBox.information(self, 
+                                    "Preprocessing Complete", 
+                                    f"Successfully preprocessed {len(self.prepared_patches)} annotations.")
         except Exception as e:
             QMessageBox.critical(self, "Error", f"Failed to preprocess annotations: {str(e)}")
         finally:
@@ -158,14 +159,17 @@ class Classify(Base):
         Perform only the inference step.
         """
         if not self.prepared_patches:
-            QMessageBox.warning(self, "Warning", "No preprocessed annotations found. Please run preprocessing first.")
+            QMessageBox.warning(self, 
+                                "Warning", 
+                                "No preprocessed annotations found. Please run preprocessing first.")
             return
             
         QApplication.setOverrideCursor(Qt.WaitCursor)
         try:
             self.batch_inference()
-            QMessageBox.information(self, "Inference Complete", 
-                                  "Batch inference completed successfully.")
+            QMessageBox.information(self, 
+                                    "Inference Complete", 
+                                   "Batch inference completed successfully.")
         except Exception as e:
             QMessageBox.critical(self, "Error", f"Failed to perform inference: {str(e)}")
         finally:
@@ -235,8 +239,8 @@ class Classify(Base):
                                    key=attrgetter('image_path'))
 
         try:
-            # Use ThreadPoolExecutor for parallel processing with -4 worker threads
-            with ThreadPoolExecutor(max_workers=os.cpu_count() - 4) as executor:
+            # Use ThreadPoolExecutor for parallel processing
+            with ThreadPoolExecutor(max_workers=os.cpu_count() // 2) as executor:
                 # Dictionary to track futures and their corresponding image paths
                 futures = {}
                 
@@ -257,15 +261,19 @@ class Classify(Base):
 
                 # Process completed futures as they finish
                 for future in as_completed(futures):
-                    # Get cropped patches from completed task
-                    cropped = future.result()
-                    # Add cropped patches to prepared patches list
-                    self.prepared_patches.extend(cropped)
-                    # Update progress bar after each image is processed
-                    progress_bar.update_progress()
+                    try:
+                        # Get cropped patches from completed task
+                        cropped = future.result()
+                        # Add cropped patches to prepared patches list
+                        self.prepared_patches.extend(cropped)
+                    except Exception as exc:
+                        print(f"{futures[future]} generated an exception: {exc}")
+                    finally:
+                        # Update progress bar after each image is processed
+                        progress_bar.update_progress()
 
-        except Exception as exc:
-            print(f"{futures[future]} generated an exception: {exc}")
+        except Exception as e:
+            print(f"{futures[future]} generated an exception: {e}")
 
         finally:
             # Restore the cursor
@@ -295,18 +303,20 @@ class Classify(Base):
         groups = groupby(sorted(self.prepared_patches, key=attrgetter('image_path')),
                         key=attrgetter('image_path'))
 
-        try:
-            # Make predictions on each image's annotations
-            for path, patches in groups:
+        # Make predictions on each image's annotations
+        for path, patches in groups:
+            try:
                 print(f"\nMaking predictions on {path}")
                 self.deploy_model_dialog.predict(inputs=list(patches))
+                
+            except Exception as e:
+                print(f"Failed to make predictions on {path}: {e}")
+                continue
+            
+            finally: 
                 progress_bar.update_progress()
 
-        except Exception as e:
-            raise e
-
-        finally:
-            # Restore the cursor
-            QApplication.restoreOverrideCursor()
-            progress_bar.stop_progress()
-            progress_bar.close()
+        QApplication.restoreOverrideCursor()
+        progress_bar.finish_progress()
+        progress_bar.stop_progress()
+        progress_bar.close()
