@@ -1,11 +1,8 @@
 import warnings
-warnings.filterwarnings("ignore", category=DeprecationWarning)
-warnings.filterwarnings("ignore", category=UserWarning)
 
-import gc
-import json
 import os
-import random
+import gc
+import ujson as json
 
 from PyQt5.QtGui import QColor
 from PyQt5.QtWidgets import (QFileDialog, QMessageBox, QVBoxLayout, QLabel, QDialog,
@@ -15,6 +12,9 @@ from torch.cuda import empty_cache
 
 from coralnet_toolbox.Icons import get_icon
 
+warnings.filterwarnings("ignore", category=DeprecationWarning)
+warnings.filterwarnings("ignore", category=UserWarning)
+
 
 # ----------------------------------------------------------------------------------------------------------------------
 # Classes
@@ -32,6 +32,7 @@ class Base(QDialog):
         super().__init__(parent)
         self.main_window = main_window
         self.label_window = main_window.label_window
+        self.image_window = main_window.image_window
         self.annotation_window = main_window.annotation_window
         self.sam_dialog = None
 
@@ -40,12 +41,14 @@ class Base(QDialog):
         self.resize(400, 325)
 
         # Initialize variables
-        self.task = None
         self.imgsz = 1024
         self.iou_thresh = 0.20
         self.uncertainty_thresh = 0.30
         self.area_thresh_min = 0.00
         self.area_thresh_max = 0.40
+
+        self.task = None
+        self.max_detect = 300
         self.model_path = None
         self.loaded_model = None
         self.class_names = []
@@ -59,6 +62,8 @@ class Base(QDialog):
         self.setup_labels_layout()
         # Setup parameters layout
         self.setup_parameters_layout()
+        # Setup SAM layout
+        self.setup_sam_layout()
         # Setup the button layout
         self.setup_buttons_layout()
         # Setup the status layout
@@ -99,6 +104,9 @@ class Base(QDialog):
     def setup_parameters_layout(self):
         raise NotImplementedError("Subclasses must implement this method")
 
+    def setup_sam_layout(self):
+        raise NotImplementedError("Subclasses must implement this method")
+
     def setup_buttons_layout(self):
         """
         Set up the buttons layout in a 2x2 grid
@@ -204,10 +212,10 @@ class Base(QDialog):
 
         :return: Boolean indicating whether the SAM model is deployed
         """
-        if not hasattr(self.main_window, 'sam_deploy_model_dialog'):
+        if not hasattr(self.main_window, 'sam_deploy_predictor_dialog'):
             return False
 
-        self.sam_dialog = self.main_window.sam_deploy_model_dialog
+        self.sam_dialog = self.main_window.sam_deploy_predictor_dialog
 
         if not self.sam_dialog.loaded_model:
             self.use_sam_dropdown.setCurrentText("False")
@@ -303,6 +311,16 @@ class Base(QDialog):
                 f"until added manually:\n{missing_labels_str}"
             )
 
+    def add_labels_to_label_window(self):
+        """
+        Add labels to the label window based on the class mapping.
+        """
+        if self.class_mapping:
+            for label in self.class_mapping.values():
+                self.label_window.add_label_if_not_exists(label['short_label_code'],
+                                                          label['long_label_code'],
+                                                          QColor(*label['color']))
+
     def handle_missing_class_mapping(self):
         """
         Handle the case when the class mapping file is missing.
@@ -314,31 +332,16 @@ class Base(QDialog):
         if reply == QMessageBox.Yes:
             self.create_generic_labels()
 
-    def add_labels_to_label_window(self):
-        """
-        Add labels to the label window based on the class mapping.
-        """
-        if self.class_mapping:
-            for label in self.class_mapping.values():
-                self.label_window.add_label_if_not_exists(label['short_label_code'],
-                                                          label['long_label_code'],
-                                                          QColor(*label['color']))
-
     def create_generic_labels(self):
         """
         Create generic labels for the given class names
         """
         for class_name in self.class_names:
-            r = random.randint(0, 255)
-            g = random.randint(0, 255)
-            b = random.randint(0, 255)
             # Create the label in the label window
-            self.label_window.add_label_if_not_exists(
+            label = self.label_window.add_label_if_not_exists(
                 class_name,
                 class_name,
-                QColor(r, g, b)
             )
-            label = self.label_window.get_label_by_short_code(class_name)
             self.class_mapping[class_name] = label.to_dict()
 
     def predict(self, inputs):
