@@ -1,11 +1,8 @@
 import warnings
 
-warnings.filterwarnings("ignore", category=DeprecationWarning)
-warnings.filterwarnings("ignore", category=UserWarning)
-
 import os
-import json
 import random
+import ujson as json
 
 from PyQt5.QtCore import Qt
 from PyQt5.QtGui import QBrush, QColor
@@ -17,8 +14,14 @@ from PyQt5.QtWidgets import (QFileDialog, QApplication, QMessageBox, QCheckBox,
 from coralnet_toolbox.Annotations.QtRectangleAnnotation import RectangleAnnotation
 from coralnet_toolbox.Annotations.QtPolygonAnnotation import PolygonAnnotation
 from coralnet_toolbox.Annotations.QtPatchAnnotation import PatchAnnotation
+
+from coralnet_toolbox.QtProgressBar import ProgressBar
+
 from coralnet_toolbox.Icons import get_icon
 
+warnings.filterwarnings("ignore", category=DeprecationWarning)
+warnings.filterwarnings("ignore", category=UserWarning)
+
 
 # ----------------------------------------------------------------------------------------------------------------------
 # Classes
@@ -39,7 +42,7 @@ class Base(QDialog):
         self.annotation_window = main_window.annotation_window
         self.image_window = main_window.image_window
 
-        self.resize(1000, 600)
+        self.resize(800, 800)
         self.setWindowIcon(get_icon("coral.png"))
         self.setWindowTitle("Export Dataset")
 
@@ -57,12 +60,17 @@ class Base(QDialog):
 
         # Setup the layout
         self.setup_info_layout()
+        # Setup the output layout
         self.setup_output_layout()
+        # Setup the ratio layout
         self.setup_ratio_layout()
-        self.setup_annotation_layout()
-        self.setup_options_layout()
+        # Setup the data selection layout
+        self.setup_data_selection_layout()
+        # Setup the table layout
         self.setup_table_layout()
+        # Setup the status layout
         self.setup_status_layout()
+        # Setup the button layout
         self.setup_button_layout()
 
     def showEvent(self, event):
@@ -84,33 +92,24 @@ class Base(QDialog):
         """
         raise NotImplementedError("Subclasses must implement this method.")
 
-    def setup_layout(self):
-        """
-        Set up the layout for the ExportDatasetDialog.
-        """
-        # Ready Status
-        self.ready_label = QLabel()
-        self.layout.addWidget(self.ready_label)
-
-        # Shuffle Button
-        self.shuffle_button = QPushButton("Shuffle")
-        self.shuffle_button.clicked.connect(self.update_summary_statistics)
-        self.layout.addWidget(self.shuffle_button)
-
     def setup_output_layout(self):
         """Setup output directory layout."""
         group_box = QGroupBox("Output Parameters")
         layout = QFormLayout()
 
-        # Dataset Name and Output Directory
+        # Dataset Name
         self.dataset_name_edit = QLineEdit()
+        layout.addRow("Dataset Name:", self.dataset_name_edit)
+
+        # Output Directory with Browse button on same line
+        output_layout = QHBoxLayout()
         self.output_dir_edit = QLineEdit()
         self.output_dir_button = QPushButton("Browse...")
         self.output_dir_button.clicked.connect(self.browse_output_dir)
+        output_layout.addWidget(self.output_dir_edit)
+        output_layout.addWidget(self.output_dir_button)
 
-        layout.addRow("Dataset Name:", self.dataset_name_edit)
-        layout.addRow("Output Directory:", self.output_dir_edit)
-        layout.addRow(self.output_dir_button)
+        layout.addRow("Output Directory:", output_layout)
 
         group_box.setLayout(layout)
         self.layout.addWidget(group_box)
@@ -143,56 +142,56 @@ class Base(QDialog):
         layout.addWidget(QLabel("Test Ratio:"))
         layout.addWidget(self.test_ratio_spinbox)
 
-        self.train_ratio_spinbox.valueChanged.connect(self.update_summary_statistics)
-        self.val_ratio_spinbox.valueChanged.connect(self.update_summary_statistics)
-        self.test_ratio_spinbox.valueChanged.connect(self.update_summary_statistics)
-
         group_box.setLayout(layout)
         self.layout.addWidget(group_box)
 
-    def setup_annotation_layout(self):
-        """Setup the annotation type checkboxes layout."""
+    def setup_data_selection_layout(self):
+        """Setup the layout for data selection options in a horizontal arrangement."""
+        options_layout = QHBoxLayout()
+
+        # Create and add the group boxes
+        annotation_types_group = self.create_annotation_layout()
+        image_options_group = self.create_image_source_layout()
+        negative_samples_group = self.create_negative_samples_layout()
+
+        options_layout.addWidget(annotation_types_group)
+        options_layout.addWidget(image_options_group)
+        options_layout.addWidget(negative_samples_group)
+
+        self.layout.addLayout(options_layout)
+
+    def create_annotation_layout(self):
+        """Creates the annotation type checkboxes layout group box."""
         group_box = QGroupBox("Annotation Types")
-        layout = QHBoxLayout()
+        layout = QVBoxLayout()
 
         self.include_patches_checkbox = QCheckBox("Include Patch Annotations")
         self.include_rectangles_checkbox = QCheckBox("Include Rectangle Annotations")
         self.include_polygons_checkbox = QCheckBox("Include Polygon Annotations")
 
-        # Connect checkbox signals
-        self.include_patches_checkbox.stateChanged.connect(self.update_summary_statistics)
-        self.include_rectangles_checkbox.stateChanged.connect(self.update_summary_statistics)
-        self.include_polygons_checkbox.stateChanged.connect(self.update_summary_statistics)
-
         layout.addWidget(self.include_patches_checkbox)
         layout.addWidget(self.include_rectangles_checkbox)
         layout.addWidget(self.include_polygons_checkbox)
 
         group_box.setLayout(layout)
-        self.layout.addWidget(group_box)
+        return group_box
 
-    def setup_options_layout(self):
-        """Setup the image options layout."""
-        group_box = QGroupBox("Image Options")
+    def create_image_source_layout(self):
+        """Creates the image source options layout group box."""
+        group_box = QGroupBox("Image Source")
         layout = QVBoxLayout()
 
-        # Create a button group for the image checkboxes
         self.image_options_group = QButtonGroup(self)
 
         self.all_images_radio = QRadioButton("All Images")
         self.filtered_images_radio = QRadioButton("Filtered Images")
 
-        # Add the radio buttons to the button group
         self.image_options_group.addButton(self.all_images_radio)
         self.image_options_group.addButton(self.filtered_images_radio)
-
-        # Ensure only one radio button can be checked at a time
         self.image_options_group.setExclusive(True)
 
-        # Set the default radio button
         self.all_images_radio.setChecked(True)
 
-        # Connect radio button signals
         self.all_images_radio.toggled.connect(self.update_image_selection)
         self.filtered_images_radio.toggled.connect(self.update_image_selection)
 
@@ -200,7 +199,32 @@ class Base(QDialog):
         layout.addWidget(self.filtered_images_radio)
 
         group_box.setLayout(layout)
-        self.layout.addWidget(group_box)
+        return group_box
+
+    def create_negative_samples_layout(self):
+        """Creates the negative sample options layout group box."""
+        group_box = QGroupBox("Negative Samples")
+        layout = QVBoxLayout()
+
+        self.negative_samples_group = QButtonGroup(self)
+
+        self.include_negatives_radio = QRadioButton("Include Negatives")
+        self.exclude_negatives_radio = QRadioButton("Exclude Negatives")
+
+        self.negative_samples_group.addButton(self.include_negatives_radio)
+        self.negative_samples_group.addButton(self.exclude_negatives_radio)
+        self.negative_samples_group.setExclusive(True)
+
+        self.exclude_negatives_radio.setChecked(True)
+
+        # Connect to update stats when changed. Only one needed for the group.
+        self.include_negatives_radio.toggled.connect(self.update_summary_statistics)
+
+        layout.addWidget(self.include_negatives_radio)
+        layout.addWidget(self.exclude_negatives_radio)
+
+        group_box.setLayout(layout)
+        return group_box
 
     def setup_table_layout(self):
         """Setup the label counts table layout."""
@@ -217,7 +241,6 @@ class Base(QDialog):
                                                            "Test",
                                                            "Images"])
         # Connect
-        self.label_counts_table.cellChanged.connect(self.update_summary_statistics)
         layout.addWidget(self.label_counts_table)
 
         group_box.setLayout(layout)
@@ -238,6 +261,11 @@ class Base(QDialog):
         """Setup the button layout."""
         button_layout = QHBoxLayout()
 
+        # Add Refresh button
+        self.refresh_button = QPushButton("Refresh")
+        self.refresh_button.clicked.connect(self.update_summary_statistics)
+        button_layout.addWidget(self.refresh_button)
+
         # Add Shuffle button
         self.shuffle_button = QPushButton("Shuffle")
         self.shuffle_button.clicked.connect(self.update_summary_statistics)
@@ -257,18 +285,6 @@ class Base(QDialog):
     def update_annotation_type_checkboxes(self):
         raise NotImplementedError("Method must be implemented in the subclass.")
 
-    def on_include_checkbox_state_changed(self, state):
-        """
-        Handle the state change event of the include checkboxes.
-
-        Args:
-            state: The new state of the checkbox.
-        """
-        if state == Qt.Checked:
-            self.update_summary_statistics()
-        elif state == Qt.Unchecked:
-            self.update_summary_statistics()
-
     def set_cell_color(self, row, column, color):
         """
         Set the background color of a cell in the label counts table.
@@ -363,7 +379,8 @@ class Base(QDialog):
 
         # Filter annotations based on the selected image option
         if self.filtered_images_radio.isChecked():
-            annotations = [a for a in annotations if a.image_path in self.image_window.filtered_image_paths]
+            # Use the table_model's filtered_paths instead of filtered_image_paths
+            annotations = [a for a in annotations if a.image_path in self.image_window.table_model.filtered_paths]
 
         return annotations
 
@@ -371,13 +388,6 @@ class Base(QDialog):
         """
         Populate the class filter list with labels and their counts.
         """
-        try:
-            # Temporarily disconnect the cellChanged signal
-            self.label_counts_table.cellChanged.disconnect()
-        except TypeError:
-            # Ignore the error if the signal was not connected
-            pass
-
         # Set the row count to 0
         self.label_counts_table.setRowCount(0)
 
@@ -414,7 +424,6 @@ class Base(QDialog):
         for label, count in sorted_label_counts:
             include_checkbox = QCheckBox()
             include_checkbox.setChecked(True)
-            include_checkbox.stateChanged.connect(self.on_include_checkbox_state_changed)
             label_item = QTableWidgetItem(label)
             anno_count = QTableWidgetItem(str(count))
             train_item = QTableWidgetItem("0")
@@ -433,10 +442,6 @@ class Base(QDialog):
 
             row += 1
 
-        # Reconnect the cellChanged signal
-        self.label_counts_table.cellChanged.connect(
-            self.update_summary_statistics)
-
     def split_data(self):
         """
         Split the data by images based on the specified ratios.
@@ -445,7 +450,17 @@ class Base(QDialog):
         self.val_ratio = self.val_ratio_spinbox.value()
         self.test_ratio = self.test_ratio_spinbox.value()
 
-        images = self.image_window.image_paths
+        # Get images, either filtered or all depending on radio button selection
+        if self.filtered_images_radio.isChecked():
+            images = self.image_window.table_model.filtered_paths
+        else:
+            images = self.image_window.raster_manager.image_paths
+
+        # If "Exclude Negatives" is checked, only use images that have selected annotations.
+        if self.exclude_negatives_radio.isChecked():
+            image_paths_with_annotations = {a.image_path for a in self.selected_annotations}
+            images = [img for img in images if img in image_paths_with_annotations]
+
         random.shuffle(images)
 
         train_split = int(len(images) * self.train_ratio)
@@ -474,34 +489,64 @@ class Base(QDialog):
 
     def check_label_distribution(self):
         """
-        Check the label distribution in the splits to ensure all labels are present.
-
+        Check the label distribution in the splits to ensure all labels are present,
+        and only allow specific split combinations.
+    
         Returns:
-            bool: True if all labels are present in all splits, False otherwise.
+            bool: True if all labels are present in all splits and split config is allowed, False otherwise.
         """
         # Get the ratios from the spinboxes
         train_ratio = self.train_ratio_spinbox.value()
         val_ratio = self.val_ratio_spinbox.value()
         test_ratio = self.test_ratio_spinbox.value()
+    
+        # Only allow these split combinations:
+        # - Train only
+        # - Test only
+        # - Train/Val
+        # - Train/Val/Test
+        
+        allowed = False
 
+        # Train only
+        if train_ratio == 1.0 and val_ratio == 0 and test_ratio == 0:
+            allowed = True
+            
+        # Test only
+        elif train_ratio == 0 and val_ratio == 0 and test_ratio == 1.0:
+            allowed = True
+            
+        # Train/Val
+        elif train_ratio > 0 and val_ratio > 0 and test_ratio == 0:
+            if abs(train_ratio + val_ratio - 1.0) < 1e-9:
+                allowed = True
+                
+        # Train/Val/Test
+        elif train_ratio > 0 and val_ratio > 0 and test_ratio > 0:
+            if abs(train_ratio + val_ratio + test_ratio - 1.0) < 1e-9:
+                allowed = True
+
+        if not allowed:
+            return False
+    
         # Initialize dictionaries to store label counts for each split
         train_label_counts = {}
         val_label_counts = {}
         test_label_counts = {}
-
+    
         # Count annotations for each label in each split
         for annotation in self.train_annotations:
             label = annotation.label.short_label_code
             train_label_counts[label] = train_label_counts.get(label, 0) + 1
-
+    
         for annotation in self.val_annotations:
             label = annotation.label.short_label_code
             val_label_counts[label] = val_label_counts.get(label, 0) + 1
-
+    
         for annotation in self.test_annotations:
             label = annotation.label.short_label_code
             test_label_counts[label] = test_label_counts.get(label, 0) + 1
-
+    
         # Check the conditions for each split
         for label in self.selected_labels:
             if train_ratio > 0 and (label not in train_label_counts or train_label_counts[label] == 0):
@@ -510,7 +555,7 @@ class Base(QDialog):
                 return False
             if test_ratio > 0 and (label not in test_label_counts or test_label_counts[label] == 0):
                 return False
-
+    
         # Additional checks to ensure no empty splits
         if train_ratio > 0 and len(self.train_annotations) == 0:
             return False
@@ -518,15 +563,7 @@ class Base(QDialog):
             return False
         if test_ratio > 0 and len(self.test_annotations) == 0:
             return False
-
-        # Allow creation of dataset if
-        if train_ratio >= 0 and val_ratio >= 0 and test_ratio >= 0:
-            return True
-        if train_ratio >= 0 and val_ratio >= 0 and test_ratio == 0:
-            return True
-        if train_ratio == 0 and val_ratio == 0 and test_ratio == 1:
-            return True
-
+    
         return True
 
     def update_image_selection(self):
@@ -543,10 +580,13 @@ class Base(QDialog):
         if self.updating_summary_statistics:
             return
 
-        self.updating_summary_statistics = True
+        # Make cursor busy
+        QApplication.setOverrideCursor(Qt.WaitCursor)
+        progress_bar = ProgressBar(self, "Updating Summary Statistics")
+        progress_bar.show()
+        progress_bar.start_progress(100)
 
-        # Split the data by images
-        self.split_data()
+        self.updating_summary_statistics = True
 
         # Selected labels based on user's selection
         self.selected_labels = []
@@ -558,6 +598,9 @@ class Base(QDialog):
 
         # Filter annotations based on the selected annotation types and current tab
         self.selected_annotations = self.filter_annotations()
+        
+        # Split the data by images
+        self.split_data()
 
         # Split the data by annotations
         self.determine_splits()
@@ -600,6 +643,12 @@ class Base(QDialog):
 
         self.updating_summary_statistics = False
 
+        # Restore the cursor to the default cursor
+        QApplication.restoreOverrideCursor()
+        progress_bar.finish_progress()
+        progress_bar.close()
+        progress_bar = None
+
     def is_ready(self):
         """Check if the dataset is ready to be created."""
         # Extract the input values, store them in the class variables
@@ -608,26 +657,34 @@ class Base(QDialog):
         self.train_ratio = self.train_ratio_spinbox.value()
         self.val_ratio = self.val_ratio_spinbox.value()
         self.test_ratio = self.test_ratio_spinbox.value()
-
-        if not self.ready_status:
-            QMessageBox.warning(self,
-                                "Dataset Not Ready",
-                                "Not all labels are present in all sets.\n"
-                                "Please adjust your selections or sample more data.")
-            return False
-
+        
+        # Check that all fields are filled
         if not self.dataset_name or not self.output_dir:
             QMessageBox.warning(self,
                                 "Input Error",
                                 "All fields must be filled.")
             return False
-
+        
+        # Check that the ratios sum to 1.0
         if abs(self.train_ratio + self.val_ratio + self.test_ratio - 1.0) > 1e-9:
             QMessageBox.warning(self,
                                 "Input Error",
                                 "Train, Validation, and Test ratios must sum to 1.0")
             return False
 
+        if not self.ready_status:
+            reply = QMessageBox.question(
+                self,
+                "Dataset Not Ready",
+                "Not all selected labels are present in all sets.\n"
+                "Are you sure you want to proceed?",
+                QMessageBox.Yes | QMessageBox.No
+            )
+            if reply == QMessageBox.Yes:
+                return True
+            else:
+                return False
+
         return True
 
     def accept(self):
@@ -667,19 +724,21 @@ class Base(QDialog):
             class_mapping = self.get_class_mapping()
             self.save_class_mapping_json(class_mapping, output_dir_path)
 
-        # Set the cursor to waiting (busy) cursor
-        QApplication.setOverrideCursor(Qt.WaitCursor)
-        self.create_dataset(output_dir_path)
-        # Restore the cursor to the default cursor
-        QApplication.restoreOverrideCursor()
+        try:
+            # Create the dataset
+            self.create_dataset(output_dir_path)
 
-        QMessageBox.information(self,
-                                "Dataset Created",
-                                "Dataset has been successfully created.")
+            QMessageBox.information(self,
+                                    "Dataset Created",
+                                    "Dataset has been successfully created.")
+        
+        except Exception as e:
+            QMessageBox.critical(self, "Failed to Create Dataset", f"{e}")
+            
         super().accept()
 
     def create_dataset(self, output_dir_path):
         raise NotImplementedError("Method must be implemented in the subclass.")
 
     def process_annotations(self, annotations, split_dir, split):
-        raise NotImplementedError("Method must be implemented in the subclass.")
+        raise NotImplementedError("Method must be implemented in the subclass.")
\ No newline at end of file
