@@ -1,4 +1,5 @@
-from PyQt5.QtWidgets import (QGroupBox, QVBoxLayout, QLabel)
+from PyQt5.QtCore import Qt
+from PyQt5.QtWidgets import (QGroupBox, QLabel, QFormLayout, QComboBox)
 
 from coralnet_toolbox.MachineLearning.ImportDataset.QtBase import Base
 
@@ -15,18 +16,15 @@ class Detect(Base):
     def __init__(self, main_window, parent=None):
         super().__init__(main_window, parent)        
         self.setWindowTitle('Import Object Detection Dataset')
+        self.task = 'detect'
         
     def setup_info_layout(self):
-        """Setup the info layout"""
-        group_box = QGroupBox("Information")
-        layout = QVBoxLayout()
-        
-        # Create a QLabel with explanatory text and hyperlink
-        info_label = QLabel("Import a YOLO-formatted Detection dataset.")
-        
-        info_label.setOpenExternalLinks(True)
-        info_label.setWordWrap(True)
-        layout.addWidget(info_label)
-        
-        group_box.setLayout(layout)
-        self.layout.addWidget(group_box)
\ No newline at end of file
+        group_box = QGroupBox("Import Options")
+        layout = QFormLayout(group_box)
+
+        import_as_label = QLabel("Import as:")
+        self.import_as_combo = QComboBox()
+        self.import_as_combo.addItems(["Rectangles (Default)", "Polygons"])
+        layout.addRow(import_as_label, self.import_as_combo)
+
+        self.layout.insertWidget(0, group_box)
\ No newline at end of file
