@@ -10,19 +10,16 @@ import torch
 from torch.cuda import empty_cache
 from ultralytics.models.fastsam import FastSAMPredictor
 
-from superqt import QRangeSlider
 from PyQt5.QtCore import Qt
 from PyQt5.QtWidgets import (QApplication, QComboBox, QDialog, QFormLayout, QHBoxLayout,
                              QLabel, QMessageBox, QPushButton, QSlider, QSpinBox,
                              QVBoxLayout, QGroupBox)
 
-
-from coralnet_toolbox.ResultsProcessor import ResultsProcessor
+from coralnet_toolbox.Results import ResultsProcessor
+from coralnet_toolbox.Results import MapResults
 
 from coralnet_toolbox.QtProgressBar import ProgressBar
 
-from coralnet_toolbox.utilities import open_image
-
 from coralnet_toolbox.Icons import get_icon
 
 
@@ -39,7 +36,7 @@ class DeployGeneratorDialog(QDialog):
 
     def __init__(self, main_window, parent=None):
         """
-        Initialize the AutoDistillDeployModelDialog.
+        Initialize the DeployGeneratorDialog.
 
         Args:
             main_window: The main application window.
@@ -48,10 +45,11 @@ class DeployGeneratorDialog(QDialog):
         super().__init__(parent)
         self.main_window = main_window
         self.label_window = main_window.label_window
+        self.image_window = main_window.image_window
         self.annotation_window = main_window.annotation_window
         self.sam_dialog = None
 
-        self.setWindowIcon(get_icon("sam.png"))
+        self.setWindowIcon(get_icon("wizard.png"))
         self.setWindowTitle("FastSAM Generator (Ctrl + 4)")
         self.resize(400, 325)
 
@@ -66,7 +64,7 @@ class DeployGeneratorDialog(QDialog):
         self.max_detect = 300
         self.loaded_model = None
         self.model_path = None
-        self.class_mapping = {0: 'Review'}
+        self.class_mapping = None
 
         # Create the layout
         self.layout = QVBoxLayout(self)
@@ -77,6 +75,10 @@ class DeployGeneratorDialog(QDialog):
         self.setup_models_layout()
         # Setup the parameter layout
         self.setup_parameters_layout()
+        # Setup the detect as layout
+        self.detect_as_layout()
+        # Setup the SAM layout
+        self.setup_sam_layout()
         # Setup the buttons layout
         self.setup_buttons_layout()
         # Setup the status layout
@@ -93,6 +95,7 @@ class DeployGeneratorDialog(QDialog):
         self.initialize_uncertainty_threshold()
         self.initialize_iou_threshold()
         self.initialize_area_threshold()
+        self.update_detect_as_combo()
 
     def setup_info_layout(self):
         """
@@ -144,6 +147,18 @@ class DeployGeneratorDialog(QDialog):
         group_box = QGroupBox("Parameters")
         layout = QFormLayout()
 
+        # Task dropdown
+        self.use_task_dropdown = QComboBox()
+        self.use_task_dropdown.addItems(["detect", "segment"])
+        self.use_task_dropdown.currentIndexChanged.connect(self.update_task)
+        layout.addRow("Task:", self.use_task_dropdown)
+
+        # Max detections spinbox
+        self.max_detections_spinbox = QSpinBox()
+        self.max_detections_spinbox.setRange(1, 10000)
+        self.max_detections_spinbox.setValue(self.max_detect)
+        layout.addRow("Max Detections:", self.max_detections_spinbox)
+
         # Resize image dropdown
         self.resize_image_dropdown = QComboBox()
         self.resize_image_dropdown.addItems(["True", "False"])
@@ -153,10 +168,9 @@ class DeployGeneratorDialog(QDialog):
 
         # Image size control
         self.imgsz_spinbox = QSpinBox()
-        self.imgsz_spinbox.setRange(512, 65536)
+        self.imgsz_spinbox.setRange(1024, 65536)
         self.imgsz_spinbox.setSingleStep(1024)
         self.imgsz_spinbox.setValue(self.imgsz)
-        self.imgsz_spinbox.setEnabled(False)  # Grey out the dropdown
         layout.addRow("Image Size (imgsz):", self.imgsz_spinbox)
 
         # Uncertainty threshold controls
@@ -187,38 +201,52 @@ class DeployGeneratorDialog(QDialog):
         min_val, max_val = self.main_window.get_area_thresh()
         self.area_thresh_min = int(min_val * 100)
         self.area_thresh_max = int(max_val * 100)
-        self.area_threshold_slider = QRangeSlider(Qt.Horizontal)
-        self.area_threshold_slider.setRange(0, 100)
-        self.area_threshold_slider.setValue((self.area_thresh_min, self.area_thresh_max))
-        self.area_threshold_slider.setTickPosition(QSlider.TicksBelow)
-        self.area_threshold_slider.setTickInterval(10)
-        self.area_threshold_slider.valueChanged.connect(self.update_area_label)
+        self.area_threshold_min_slider = QSlider(Qt.Horizontal)
+        self.area_threshold_min_slider.setRange(0, 100)
+        self.area_threshold_min_slider.setValue(self.area_thresh_min)
+        self.area_threshold_min_slider.setTickPosition(QSlider.TicksBelow)
+        self.area_threshold_min_slider.setTickInterval(10)
+        self.area_threshold_min_slider.valueChanged.connect(self.update_area_label)
+        self.area_threshold_max_slider = QSlider(Qt.Horizontal)
+        self.area_threshold_max_slider.setRange(0, 100)
+        self.area_threshold_max_slider.setValue(self.area_thresh_max)
+        self.area_threshold_max_slider.setTickPosition(QSlider.TicksBelow)
+        self.area_threshold_max_slider.setTickInterval(10)
+        self.area_threshold_max_slider.valueChanged.connect(self.update_area_label)
         self.area_threshold_label = QLabel(f"{self.area_thresh_min:.2f} - {self.area_thresh_max:.2f}")
-        layout.addRow("Area Threshold", self.area_threshold_slider)
+        layout.addRow("Area Threshold Min", self.area_threshold_min_slider)
+        layout.addRow("Area Threshold Max", self.area_threshold_max_slider)
         layout.addRow("", self.area_threshold_label)
 
-        # Max detections spinbox
-        self.max_detections_spinbox = QSpinBox()
-        self.max_detections_spinbox.setRange(1, 10000)
-        self.max_detections_spinbox.setValue(self.max_detect)
-        label = QLabel("Max Detections")
-        layout.addRow(label, self.max_detections_spinbox)
+        group_box.setLayout(layout)
+        self.layout.addWidget(group_box)
 
-        # Task dropdown
-        self.use_task_dropdown = QComboBox()
-        self.use_task_dropdown.addItems(["Detect", "Segment"])
-        self.use_task_dropdown.currentIndexChanged.connect(self.update_task)
-        self.use_task_dropdown.currentIndexChanged.connect(self.deactivate_model)
-        label = QLabel("Choose a task to perform")
-        layout.addRow(label, self.use_task_dropdown)
+    def detect_as_layout(self):
+        """Detect objects as layout."""
+        group_box = QGroupBox("Detect as: ")
+        layout = QFormLayout()
+
+        # Sample Label
+        self.detect_as_combo = QComboBox()
+        for label in self.label_window.labels:
+            self.detect_as_combo.addItem(label.short_label_code, label.id)
+        self.detect_as_combo.setCurrentIndex(0)
+        self.detect_as_combo.currentIndexChanged.connect(self.update_class_mapping)
+        layout.addRow("Detect as:", self.detect_as_combo)
+
+        group_box.setLayout(layout)
+        self.layout.addWidget(group_box)
+
+    def setup_sam_layout(self):
+        """Use SAM model for segmentation."""
+        group_box = QGroupBox("Use SAM Model for Creating Polygons")
+        layout = QFormLayout()
 
         # SAM dropdown
         self.use_sam_dropdown = QComboBox()
         self.use_sam_dropdown.addItems(["False", "True"])
         self.use_sam_dropdown.currentIndexChanged.connect(self.is_sam_model_deployed)
-        label = QLabel("Use Predictor for creating Polygons:")
-        label.setStyleSheet("font-weight: bold;")
-        layout.addRow(label, self.use_sam_dropdown)
+        layout.addRow("Use SAM Polygons:", self.use_sam_dropdown)
 
         group_box.setLayout(layout)
         self.layout.addWidget(group_box)
@@ -254,27 +282,36 @@ class DeployGeneratorDialog(QDialog):
         group_box.setLayout(layout)
         self.layout.addWidget(group_box)
 
+    def update_detect_as_combo(self):
+        """Update the label combo box with the current labels, preserving previous selection."""
+        # Store the previously selected index
+        previous_index = self.detect_as_combo.currentIndex() if hasattr(self, 'detect_as_combo') else 0
+
+        self.detect_as_combo.clear()
+        for label in self.label_window.labels:
+            self.detect_as_combo.addItem(label.short_label_code, label.id)
+
+        # Restore the previous selection if possible
+        if 0 <= previous_index < self.detect_as_combo.count():
+            self.detect_as_combo.setCurrentIndex(previous_index)
+        else:
+            self.detect_as_combo.setCurrentIndex(0)
+
+    def update_class_mapping(self):
+        """Update the class mapping based on the selected label."""
+        detect_as = self.detect_as_combo.currentText()
+        label = self.label_window.get_label_by_short_code(detect_as)
+        self.class_mapping = {0: label}
+
     def update_task(self):
-        """Update the task based on the dropdown selection."""
-        self.task = self.use_task_dropdown.currentText().lower()
+        """Update the task based on the dropdown selection and handle UI/model effects."""
+        self.task = self.use_task_dropdown.currentText()
 
-    def is_sam_model_deployed(self):
-        """
-        Check if the SAM model is deployed and update the checkbox state accordingly.
-
-        :return: Boolean indicating whether the SAM model is deployed
-        """
-        if not hasattr(self.main_window, 'sam_deploy_model_dialog'):
-            return False
-
-        self.sam_dialog = self.main_window.sam_deploy_model_dialog
-
-        if not self.sam_dialog.loaded_model:
-            self.use_sam_dropdown.setCurrentText("False")
-            QMessageBox.critical(self, "Error", "Please deploy the SAM model first.")
-            return False
-
-        return True
+        # Update UI elements based on task
+        if self.task == "segment":
+            # Deactivate model if one is loaded and we're switching to segment task
+            if self.loaded_model:
+                self.deactivate_model()
 
     def initialize_uncertainty_threshold(self):
         """Initialize the uncertainty threshold slider with the current value"""
@@ -291,7 +328,8 @@ class DeployGeneratorDialog(QDialog):
     def initialize_area_threshold(self):
         """Initialize the area threshold range slider"""
         current_min, current_max = self.main_window.get_area_thresh()
-        self.area_threshold_slider.setValue((int(current_min * 100), int(current_max * 100)))
+        self.area_threshold_min_slider.setValue(int(current_min * 100))
+        self.area_threshold_max_slider.setValue(int(current_max * 100))
         self.area_thresh_min = current_min
         self.area_thresh_max = current_max
 
@@ -311,7 +349,11 @@ class DeployGeneratorDialog(QDialog):
 
     def update_area_label(self):
         """Handle changes to area threshold range slider"""
-        min_val, max_val = self.area_threshold_slider.value()  # Returns tuple of values
+        min_val = self.area_threshold_min_slider.value()
+        max_val = self.area_threshold_max_slider.value()
+        if min_val > max_val:
+            min_val = max_val
+            self.area_threshold_min_slider.setValue(min_val)
         self.area_thresh_min = min_val / 100.0
         self.area_thresh_max = max_val / 100.0
         self.main_window.update_area_thresh(self.area_thresh_min, self.area_thresh_max)
@@ -322,6 +364,50 @@ class DeployGeneratorDialog(QDialog):
         self.max_detect = self.max_detections_spinbox.value()
         return self.max_detect
 
+    def is_sam_model_deployed(self):
+        """
+        Check if the SAM model is deployed and update the checkbox state accordingly.
+
+        :return: Boolean indicating whether the SAM model is deployed
+        """
+        if not hasattr(self.main_window, 'sam_deploy_predictor_dialog'):
+            return False
+
+        self.sam_dialog = self.main_window.sam_deploy_predictor_dialog
+
+        if not self.sam_dialog.loaded_model:
+            self.use_sam_dropdown.setCurrentText("False")
+            QMessageBox.critical(self, "Error", "Please deploy the SAM model first.")
+            return False
+
+        return True
+
+    def update_sam_task_state(self):
+        """
+        Centralized method to check if SAM is loaded and update task accordingly.
+        If the user has selected to use SAM, this function ensures the task is set to 'segment'.
+        Crucially, it does NOT alter the task if SAM is not selected, respecting the
+        user's choice from the 'Task' dropdown.
+        """
+        # Check if the user wants to use the SAM model
+        if self.use_sam_dropdown.currentText() == "True":
+            # SAM is requested. Check if it's actually available.
+            sam_is_available = (
+                hasattr(self, 'sam_dialog') and
+                self.sam_dialog is not None and
+                self.sam_dialog.loaded_model is not None
+            )
+
+            if sam_is_available:
+                # If SAM is wanted and available, the task must be segmentation.
+                self.task = 'segment'
+            else:
+                # If SAM is wanted but not available, revert the dropdown and do nothing else.
+                # The 'is_sam_model_deployed' function already handles showing an error message.
+                self.use_sam_dropdown.setCurrentText("False")
+
+        # If use_sam_dropdown is "False", do nothing. Let self.task be whatever the user set.
+
     def load_model(self):
         """
         Load the selected model with the current configuration.
@@ -331,16 +417,21 @@ class DeployGeneratorDialog(QDialog):
         # Show a progress bar
         progress_bar = ProgressBar(self.annotation_window, title="Loading Model")
         progress_bar.show()
+        
         try:
+            # Check if SAM is active and update task state
+            self.update_sam_task_state()
+            
             # Get selected model path
             self.model_path = self.models[self.model_combo.currentText()]
-            self.task = self.use_task_dropdown.currentText().lower()
+            self.task = self.use_task_dropdown.currentText()
 
             # Set the parameters
             overrides = dict(model=self.model_path,
                              task=self.task,
                              mode='predict',
                              save=False,
+                             retina_masks=self.task == "segment",
                              max_det=self.get_max_detections(),
                              imgsz=self.get_imgsz(),
                              conf=self.main_window.get_uncertainty_thresh(),
@@ -349,12 +440,13 @@ class DeployGeneratorDialog(QDialog):
 
             # Load the model
             self.loaded_model = FastSAMPredictor(overrides=overrides)
-            self.loaded_model.names = self.class_mapping
+            self.loaded_model.names = {0: self.class_mapping[0].short_label_code}
 
             with torch.no_grad():
                 # Run a blank through the model to initialize it
                 self.loaded_model(np.zeros((self.imgsz, self.imgsz, 3), dtype=np.uint8))
 
+            progress_bar.finish_progress()
             self.status_bar.setText(f"Model loaded: {self.model_path}")
             QMessageBox.information(self, "Model Loaded", "Model loaded successfully")
 
@@ -368,9 +460,6 @@ class DeployGeneratorDialog(QDialog):
             progress_bar.stop_progress()
             progress_bar.close()
 
-        # Exit the dialog box
-        self.accept()
-
     def get_imgsz(self):
         """Get the image size for the model."""
         self.imgsz = self.imgsz_spinbox.value()
@@ -400,7 +489,14 @@ class DeployGeneratorDialog(QDialog):
             # Predict only the current image
             image_paths = [self.annotation_window.current_image_path]
 
+        # Make cursor busy
         QApplication.setOverrideCursor(Qt.WaitCursor)
+
+        # Start the progress bar
+        progress_bar = ProgressBar(self.annotation_window, title="Prediction Workflow")
+        progress_bar.show()
+        progress_bar.start_progress(len(image_paths))
+
         try:
             for image_path in image_paths:
                 inputs = self._get_inputs(image_path)
@@ -408,28 +504,34 @@ class DeployGeneratorDialog(QDialog):
                     continue
 
                 results = self._apply_model(inputs)
-                results = self._update_results(results, image_path)
                 results = self._apply_sam(results, image_path)
-                results = self._apply_tile_postprocessing(results)
-                self._process_results(results_processor, results)
+                self._process_results(results_processor, results, image_path)
+
+                # Update the progress bar
+                progress_bar.update_progress()
+
         except Exception as e:
             print("An error occurred during prediction:", e)
         finally:
             QApplication.restoreOverrideCursor()
+            progress_bar.finish_progress()
+            progress_bar.stop_progress()
+            progress_bar.close()
 
         gc.collect()
         empty_cache()
 
     def _get_inputs(self, image_path):
         """Get the inputs for the model prediction."""
-        # Check if tile inference tool is enabled
-        if self.main_window.tile_inference_tool_action.isChecked():
-            inputs = self.main_window.tile_processor.make_crops(self.loaded_model, image_path)
-            if not inputs:
-                return None
+        raster = self.image_window.raster_manager.get_raster(image_path)
+        if self.annotation_window.get_selected_tool() != "work_area":
+            # Use the image path
+            work_areas_data = [raster.image_path]
         else:
-            inputs = image_path
-        return inputs
+            # Get the work areas
+            work_areas_data = raster.get_work_areas_data()
+
+        return work_areas_data
 
     def _apply_model(self, inputs):
         """Apply the model to the inputs."""
@@ -438,48 +540,125 @@ class DeployGeneratorDialog(QDialog):
         self.loaded_model.iou = self.main_window.get_iou_thresh()
         self.loaded_model.max_det = self.get_max_detections()
 
-        # Make predictions
-        with torch.no_grad():
-            results = self.loaded_model(inputs)
-            gc.collect()
-            empty_cache()
+        # Start the progress bar
+        progress_bar = ProgressBar(self.annotation_window, title="Making Predictions")
+        progress_bar.show()
+        progress_bar.start_progress(len(inputs))
 
-        # Return the results
-        yield results
+        results_list = []
 
-    def _update_results(self, results_generator, image_path):
-        """Update the results with the image path and class mapping."""
-        # Update the results with the image path and class mapping.
-        for results in results_generator:
-            for result in results:
-                if result:
-                    result.path = image_path
-                    result.names = self.class_mapping
+        # Process each input separately
+        for idx, input_image in enumerate(inputs):
+            # Make predictions on single image
+            with torch.no_grad():
+                results = self.loaded_model(input_image)
+                results_list.append(results)
+                # Update the progress bar
+                progress_bar.update_progress()
+                # Clean up GPU memory after each prediction
+                gc.collect()
+                empty_cache()
 
-        return results
+        # Close the progress bar
+        progress_bar.finish_progress()
+        progress_bar.stop_progress()
+        progress_bar.close()
 
-    def _apply_sam(self, results, image_path):
-        """Apply SAM to the results if enabled."""
-        # Check if SAM is enabled
-        if self.use_sam_dropdown.currentText() == "True":
-            self.task = 'segment'
-            results = self.sam_dialog.predict_from_results(results, self.class_mapping, image_path)
-        return results
+        return results_list
 
-    def _apply_tile_postprocessing(self, results):
-        """Apply tile postprocessing if needed."""
-        # Check if tile inference tool is enabled
-        if self.main_window.tile_inference_tool_action.isChecked():
-            results = self.main_window.tile_processor.detect_them(results, self.task == 'segment')
-        return results
+    def _apply_sam(self, results_list, image_path):
+        """Apply SAM to the results if needed."""
+        # Check if SAM model is deployed and loaded
+        self.update_sam_task_state()
+        if self.task != 'segment':
+            return results_list
+        
+        if not self.sam_dialog or self.use_sam_dropdown.currentText() == "False":
+            # If SAM is not deployed or not selected, return the results as is
+            return results_list
 
-    def _process_results(self, results_processor, results):
+        if self.sam_dialog.loaded_model is None:
+            # If SAM is not loaded, ensure we do not use it accidentally
+            self.task = 'detect'
+            self.use_sam_dropdown.setCurrentText("False")
+            return results_list
+
+        # Make cursor busy
+        QApplication.setOverrideCursor(Qt.WaitCursor)
+        progress_bar = ProgressBar(self.annotation_window, title="Predicting with SAM")
+        progress_bar.show()
+        progress_bar.start_progress(len(results_list))
+
+        updated_results = []
+
+        for idx, results in enumerate(results_list):
+            # Each Results is a list (within the results_list, [[], ]
+            if results:
+                # Run it rough the SAM model
+                results = self.sam_dialog.predict_from_results(results, image_path)
+                updated_results.append(results)
+
+            # Update the progress bar
+            progress_bar.update_progress()
+
+        # Make cursor normal
+        QApplication.restoreOverrideCursor()
+        progress_bar.finish_progress()
+        progress_bar.stop_progress()
+        progress_bar.close()
+
+        return updated_results
+
+    def _process_results(self, results_processor, results_list, image_path):
         """Process the results using the result processor."""
-        # Process the segmentations
-        if self.task.lower() == 'segment' or self.use_sam_dropdown.currentText() == "True":
-            results_processor.process_segmentation_results(results)
+        # Get the raster object and number of work items
+        raster = self.image_window.raster_manager.get_raster(image_path)
+        total = raster.count_work_items()
+
+        # Get the work areas (if any)
+        work_areas = raster.get_work_areas()
+
+        # Start the progress bar
+        progress_bar = ProgressBar(self.annotation_window, title="Processing Results")
+        progress_bar.show()
+        progress_bar.start_progress(total)
+
+        updated_results = []
+
+        for idx, results in enumerate(results_list):
+            # Each Results is a list (within the results_list, [[], ]
+            if results:
+                # Update path and names
+                results[0].path = image_path
+                results[0].names = {0: self.class_mapping[0].short_label_code}
+
+                # Check if the work area is valid, or the image path is being used
+                if work_areas and self.annotation_window.get_selected_tool() == "work_area":
+                    # Map results from work area to the full image
+                    results = MapResults().map_results_from_work_area(results[0], 
+                                                                      raster, 
+                                                                      work_areas[idx],
+                                                                      self.task == "segment")
+                else:
+                    results = results[0]
+
+                # Append the result object (not a list) to the updated results list
+                updated_results.append(results)
+
+                # Update the index for the next work area
+                idx += 1
+                progress_bar.update_progress()
+
+        # Process the Results
+        if self.task == 'segment' or self.use_sam_dropdown.currentText() == "True":
+            results_processor.process_segmentation_results(updated_results)
         else:
-            results_processor.process_detection_results(results)
+            results_processor.process_detection_results(updated_results)
+
+        # Close the progress bar
+        progress_bar.finish_progress()
+        progress_bar.stop_progress()
+        progress_bar.close()
 
     def deactivate_model(self):
         """
