@@ -8,7 +8,6 @@ import os
 import numpy as np
 import torch
 
-from superqt import QRangeSlider
 from PyQt5.QtCore import Qt
 from PyQt5.QtWidgets import (QApplication, QComboBox, QDialog, QFormLayout,
                              QHBoxLayout, QLabel, QMessageBox, QPushButton,
@@ -21,13 +20,16 @@ from x_segment_anything import sam_model_urls
 from torch.cuda import empty_cache
 from ultralytics.utils import ops
 
-from coralnet_toolbox.ResultsProcessor import ResultsProcessor
+from coralnet_toolbox.Results import ConvertResults
+
 from coralnet_toolbox.QtProgressBar import ProgressBar
-from coralnet_toolbox.Icons import get_icon
-from coralnet_toolbox.utilities import preprocess_image
+
+from coralnet_toolbox.utilities import rasterio_open
 from coralnet_toolbox.utilities import rasterio_to_numpy
 from coralnet_toolbox.utilities import attempt_download_asset
 
+from coralnet_toolbox.Icons import get_icon
+
 
 # ----------------------------------------------------------------------------------------------------------------------
 # Classes
@@ -41,7 +43,7 @@ class DeployPredictorDialog(QDialog):
         self.main_window = main_window
         self.annotation_window = main_window.annotation_window
 
-        self.setWindowIcon(get_icon("sam.png"))
+        self.setWindowIcon(get_icon("wizard.png"))
         self.setWindowTitle("SAM Deploy Model")
         self.resize(400, 325)
 
@@ -112,9 +114,9 @@ class DeployPredictorDialog(QDialog):
 
         # Define available models
         self.models = {
-            "RepViT-SAM": "repvit.pt",
-            "EdgeSAM": "edge_sam_3x.pt",
             "MobileSAM": "vit_t.pt",
+            "EdgeSAM": "edge_sam_3x.pt",
+            "RepViT-SAM": "repvit.pt",
             "CoralSCOP": "vit_b_coralscop.pt",
             "SAM-Base": "vit_b.pt",
             "SAM-Large": "vit_l.pt",
@@ -125,6 +127,9 @@ class DeployPredictorDialog(QDialog):
         for model_name in self.models.keys():
             self.model_combo.addItem(model_name)
 
+        models = list(self.models.keys())
+        self.model_combo.setCurrentIndex(models.index("MobileSAM"))
+
         layout.addWidget(QLabel("Select Model:"))
         layout.addWidget(self.model_combo)
 
@@ -137,6 +142,18 @@ class DeployPredictorDialog(QDialog):
         """
         group_box = QGroupBox("Parameters")
         layout = QFormLayout()
+        
+        # Output type dropdown (polygon or rectangle)
+        self.output_type_dropdown = QComboBox()
+        self.output_type_dropdown.addItems(["Polygon", "Rectangle"])
+        self.output_type_dropdown.setCurrentIndex(0)  # Default to Polygon
+        layout.addRow("Output Type:", self.output_type_dropdown)
+        
+        # Allow holes dropdown
+        self.allow_holes_dropdown = QComboBox()
+        self.allow_holes_dropdown.addItems(["True", "False"])
+        self.allow_holes_dropdown.setCurrentIndex(1)  # Default to False
+        layout.addRow("Allow Holes:", self.allow_holes_dropdown)
 
         # Resize image dropdown
         self.resize_image_dropdown = QComboBox()
@@ -147,7 +164,7 @@ class DeployPredictorDialog(QDialog):
 
         # Image size control
         self.imgsz_spinbox = QSpinBox()
-        self.imgsz_spinbox.setRange(512, 65536)
+        self.imgsz_spinbox.setRange(1024, 65536)
         self.imgsz_spinbox.setSingleStep(1024)
         self.imgsz_spinbox.setValue(self.imgsz)
         layout.addRow("Image Size (imgsz):", self.imgsz_spinbox)
@@ -180,14 +197,21 @@ class DeployPredictorDialog(QDialog):
         min_val, max_val = self.main_window.get_area_thresh()
         self.area_thresh_min = int(min_val * 100)
         self.area_thresh_max = int(max_val * 100)
-        self.area_threshold_slider = QRangeSlider(Qt.Horizontal)
-        self.area_threshold_slider.setRange(0, 100)
-        self.area_threshold_slider.setValue((self.area_thresh_min, self.area_thresh_max))
-        self.area_threshold_slider.setTickPosition(QSlider.TicksBelow)
-        self.area_threshold_slider.setTickInterval(10)
-        self.area_threshold_slider.valueChanged.connect(self.update_area_label)
+        self.area_threshold_min_slider = QSlider(Qt.Horizontal)
+        self.area_threshold_min_slider.setRange(0, 100)
+        self.area_threshold_min_slider.setValue(self.area_thresh_min)
+        self.area_threshold_min_slider.setTickPosition(QSlider.TicksBelow)
+        self.area_threshold_min_slider.setTickInterval(10)
+        self.area_threshold_min_slider.valueChanged.connect(self.update_area_label)
+        self.area_threshold_max_slider = QSlider(Qt.Horizontal)
+        self.area_threshold_max_slider.setRange(0, 100)
+        self.area_threshold_max_slider.setValue(self.area_thresh_max)
+        self.area_threshold_max_slider.setTickPosition(QSlider.TicksBelow)
+        self.area_threshold_max_slider.setTickInterval(10)
+        self.area_threshold_max_slider.valueChanged.connect(self.update_area_label)
         self.area_threshold_label = QLabel(f"{self.area_thresh_min:.2f} - {self.area_thresh_max:.2f}")
-        layout.addRow("Area Threshold", self.area_threshold_slider)
+        layout.addRow("Area Threshold Min", self.area_threshold_min_slider)
+        layout.addRow("Area Threshold Max", self.area_threshold_max_slider)
         layout.addRow("", self.area_threshold_label)
 
         group_box.setLayout(layout)
@@ -224,6 +248,14 @@ class DeployPredictorDialog(QDialog):
         group_box.setLayout(layout)
         self.layout.addWidget(group_box)
 
+    def get_allow_holes(self):
+        """Return the current setting for allowing holes."""
+        return self.allow_holes_dropdown.currentText() == "True"
+    
+    def get_output_type(self):
+        """Return the current setting for output type."""
+        return self.output_type_dropdown.currentText()
+    
     def initialize_uncertainty_threshold(self):
         """Initialize the uncertainty threshold slider with the current value"""
         current_value = self.main_window.get_uncertainty_thresh()
@@ -239,7 +271,8 @@ class DeployPredictorDialog(QDialog):
     def initialize_area_threshold(self):
         """Initialize the area threshold range slider"""
         current_min, current_max = self.main_window.get_area_thresh()
-        self.area_threshold_slider.setValue((int(current_min * 100), int(current_max * 100)))
+        self.area_threshold_min_slider.setValue(int(current_min * 100))
+        self.area_threshold_max_slider.setValue(int(current_max * 100))
         self.area_thresh_min = current_min
         self.area_thresh_max = current_max
 
@@ -259,7 +292,11 @@ class DeployPredictorDialog(QDialog):
 
     def update_area_label(self):
         """Handle changes to area threshold range slider"""
-        min_val, max_val = self.area_threshold_slider.value()  # Returns tuple of values
+        min_val = self.area_threshold_min_slider.value()
+        max_val = self.area_threshold_max_slider.value()
+        if min_val > max_val:
+            min_val = max_val
+            self.area_threshold_min_slider.setValue(min_val)
         self.area_thresh_min = min_val / 100.0
         self.area_thresh_max = max_val / 100.0
         self.main_window.update_area_thresh(self.area_thresh_min, self.area_thresh_max)
@@ -317,16 +354,19 @@ class DeployPredictorDialog(QDialog):
             self.loaded_model.model.to(device=self.main_window.device)
             self.loaded_model.model.eval()
 
+            progress_bar.finish_progress()
             self.status_bar.setText("Model loaded")
             QMessageBox.information(self.annotation_window, "Model Loaded", "Model loaded successfully")
 
         except Exception as e:
             QMessageBox.critical(self.annotation_window, "Error Loading Model", f"Error loading model: {e}")
 
-        progress_bar.stop_progress()
-        progress_bar.close()
-        QApplication.restoreOverrideCursor()
-        self.accept()
+        finally:
+            # Restore cursor
+            QApplication.restoreOverrideCursor()
+            # Stop the progress bar
+            progress_bar.stop_progress()
+            progress_bar.close()
 
     def resize_image(self, image):
         """
@@ -360,11 +400,7 @@ class DeployPredictorDialog(QDialog):
 
                 if image is None and image_path is not None:
                     # Open the image using rasterio
-                    image = self.main_window.image_window.rasterio_open(image_path)
-                    image = rasterio_to_numpy(image)
-
-                # Preprocess the image
-                image = preprocess_image(image)
+                    image = rasterio_to_numpy(rasterio_open(image_path))
 
                 # Save the original image
                 self.original_image = image
@@ -389,18 +425,15 @@ class DeployPredictorDialog(QDialog):
 
         except Exception as e:
             QMessageBox.critical(self.annotation_window, "Error Setting Image", f"{e}")
-            # Deactivate the SAM tool if it is active, clearing the scene
-            if self.annotation_window.tools["sam"].is_active:
-                self.annotation_window.tools["sam"].deactivate()
-
             # Deactivate the model
             self.deactivate_model()
 
         finally:
             # Ensure cleanup happens even if an error occurs
+            QApplication.restoreOverrideCursor()
+            progress_bar.finish_progress()
             progress_bar.stop_progress()
             progress_bar.close()
-            QApplication.restoreOverrideCursor()
 
     def scale_points(self, points):
         """
@@ -527,21 +560,22 @@ class DeployPredictorDialog(QDialog):
             if len(bbox) != 0:
                 bbox_coords = self.transform_bboxes(bbox)
 
+            # Request 3 masks per prompt
             masks, scores, _ = self.loaded_model.predict_torch(boxes=bbox_coords,
                                                                point_coords=point_coords,
                                                                point_labels=point_labels,
-                                                               num_multimask_outputs=1)
+                                                               num_multimask_outputs=3)
 
-            # Create a results processor
-            results_processor = ResultsProcessor(self.main_window,
-                                                 class_mapping=None,
-                                                 uncertainty_thresh=self.main_window.get_uncertainty_thresh(),
-                                                 iou_thresh=self.main_window.get_iou_thresh(),
-                                                 min_area_thresh=self.main_window.get_area_thresh_min(),
-                                                 max_area_thresh=self.main_window.get_area_thresh_max())
+            # Select the mask with the highest score for each prompt
+            best_indices = torch.argmax(scores, dim=1)  # Get indices of highest scores
+            batch_indices = torch.arange(scores.shape[0], device=scores.device)
 
-            # Post-process the results
-            results = results_processor.from_sam(masks, scores, self.original_image, self.image_path)
+            # Select the best masks and scores
+            best_masks = masks[batch_indices, best_indices].unsqueeze(1)  # Shape: (B, 1, H, W)
+            best_scores = scores[batch_indices, best_indices]  # Shape: (B)
+
+            # Post-process the results with the best masks
+            results = ConvertResults().from_sam(best_masks, best_scores, self.original_image, self.image_path)
 
         except Exception as e:
             QMessageBox.critical(self.annotation_window,
@@ -551,90 +585,26 @@ class DeployPredictorDialog(QDialog):
 
         return results
 
-    def predict_from_results(self, results_generator, class_mapping, image_path=None):
+    def predict_from_results(self, results, image_path=None):
         """
         Make predictions using the currently loaded model and grouped results.
 
         Args:
-            results_generator (generator): A generator that yields Ultralytics Results.
+            results (ultralytics.engine.Results): Results object containing the predictions.
+            image_path (str): Optional image path to override the image in the results.
         """
-        # Create a result processor with current settings.
-        result_processor = ResultsProcessor(
-            self.main_window,
-            class_mapping=class_mapping,
-            uncertainty_thresh=self.main_window.get_uncertainty_thresh(),
-            iou_thresh=self.main_window.get_iou_thresh(),
-            min_area_thresh=self.main_window.get_area_thresh_min(),
-            max_area_thresh=self.main_window.get_area_thresh_max()
-        )
+        # Get the boxes (xyxy) from the results
+        boxes = results[0].boxes.xyxy.detach().cpu().numpy()
 
-        # TODO fix progress bar
-        # Make cursor busy
-        QApplication.setOverrideCursor(Qt.WaitCursor)
-        progress_bar = ProgressBar(self.annotation_window, title="Predicting with SAM")
-        progress_bar.show()
+        # Set the image with SAM
+        self.set_image(image=results[0].orig_img, image_path=image_path)
+        # Make the predictions using the bounding boxes
+        new_results = self.predict_from_prompts(boxes, [], [])
 
-        # Process each batch from the results generator.
-        for results in results_generator:
-            # Apply filters to the results.
-            results = result_processor.apply_filters(results)
-            # Group detections by image path along with their bounding boxes and original image data.
-            group_dict = {}
+        # Update the new results with the original results attributes ()
+        results[0].update(masks=new_results.masks.data)
 
-            # Start progress bar
-            progress_bar.start_progress(len(results))
-
-            # Group the results by image path
-            for result in results:
-                if result:
-                    # Extract the detection results
-                    path, cls_id, cls_name, conf, *bbox = result_processor.extract_detection_result(result)
-                    # Get the original image
-                    orig_img = result.orig_img
-
-                    if path not in group_dict:
-                        group_dict[path] = {'bboxes': [], 'orig_img': orig_img}
-                    # Add the bounding box to the group dictionary
-                    group_dict[path]['bboxes'].append(np.array(bbox))
-
-                # Update progress bar
-                progress_bar.update_progress()
-
-            # Reset progress bar
-            progress_bar.start_progress(len(group_dict))
-
-            # Process each grouped result and yield predictions.
-            for path, group in group_dict.items():
-                try:
-                    if image_path:
-                        # Override the image path if provided
-                        path = image_path
-
-                    # Set the image in the predictor
-                    self.set_image(image=group['orig_img'], image_path=path)
-                    bboxes = np.stack(group['bboxes'])
-                    new_results = self.predict_from_prompts(bboxes, [], [])
-                    # Optionally transfer additional properties from results if available.
-                    if hasattr(results, "names"):
-                        new_results.names = results.names
-                    if hasattr(results, "boxes"):
-                        new_results.boxes = results.boxes
-
-                    yield new_results
-
-                except Exception as e:
-                    QMessageBox.critical(self.annotation_window,
-                                         "Prediction Error",
-                                         f"Error predicting for image {path}: {e}")
-                finally:
-                    progress_bar.update_progress()
-
-            # Reset progress bar
-            progress_bar.stop_progress()
-
-        # Make cursor normal
-        QApplication.restoreOverrideCursor()
-        progress_bar.close()
+        return results
 
     def deactivate_model(self):
         """
