@@ -1,6 +1,4 @@
 import warnings
-warnings.filterwarnings("ignore", category=DeprecationWarning)
-warnings.filterwarnings("ignore", category=UserWarning)
 
 import os
 import shutil
@@ -11,14 +9,19 @@ from PyQt5.QtCore import Qt
 from PyQt5.QtWidgets import (QApplication, QMessageBox, QVBoxLayout, QLabel, QDialog,
                              QDialogButtonBox, QGroupBox, QFormLayout, QLineEdit,
                              QDoubleSpinBox, QComboBox, QPushButton, QFileDialog, QSpinBox,
-                             QHBoxLayout, QWidget)
+                             QHBoxLayout)
 
-from coralnet_toolbox.QtCommon import TileSizeInput, OverlapInput, MarginInput
+from coralnet_toolbox.Common.QtTileSizeInput import TileSizeInput
+from coralnet_toolbox.Common.QtOverlapInput import OverlapInput
+from coralnet_toolbox.Common.QtMarginInput import MarginInput
 
 from coralnet_toolbox.QtProgressBar import ProgressBar
 
 from coralnet_toolbox.Icons import get_icon
 
+warnings.filterwarnings("ignore", category=DeprecationWarning)
+warnings.filterwarnings("ignore", category=UserWarning)
+
 
 # ----------------------------------------------------------------------------------------------------------------------
 # Classes
@@ -38,44 +41,37 @@ class Base(QDialog):
 
         self.setWindowIcon(get_icon("coral.png"))
         self.setWindowTitle("Tile Dataset")
-        self.resize(800, 500)
+        self.resize(600, 550)
 
         # Main vertical layout
         main_layout = QVBoxLayout(self)
 
-        # Horizontal layout for columns
-        columns_layout = QHBoxLayout()
-
-        # Create containers for left and right columns
-        left_container = QWidget()
-        right_container = QWidget()
-        left_layout = QVBoxLayout(left_container)
-        right_layout = QVBoxLayout(right_container)
-
         # Create group boxes
         self.info_group = QGroupBox()
+        self.input_dataset_group = QGroupBox()
+        self.output_dataset_group = QGroupBox()
         self.tile_config_group = QGroupBox()
-        self.dataset_group = QGroupBox()
         self.dataset_config_group = QGroupBox()
 
         # Setup layouts
         self.setup_info_layout()
+        self.setup_input_dataset_layout()
+        self.setup_output_dataset_layout()
         self.setup_tile_config_layout()
-        self.setup_dataset_layout()
         self.setup_dataset_config_layout()
 
-        # Add groups to column layouts
-        left_layout.addWidget(self.info_group)
-        left_layout.addWidget(self.tile_config_group)
-        right_layout.addWidget(self.dataset_group)
-        right_layout.addWidget(self.dataset_config_group)
+        # Add info group at the top
+        main_layout.addWidget(self.info_group)
 
-        # Add containers to columns layout
-        columns_layout.addWidget(left_container)
-        columns_layout.addWidget(right_container)
+        # Add input and output dataset groups below info
+        main_layout.addWidget(self.input_dataset_group)
+        main_layout.addWidget(self.output_dataset_group)
 
-        # Add columns to main layout
-        main_layout.addLayout(columns_layout)
+        # Create bottom row with tile config and dataset config side by side
+        bottom_row_layout = QHBoxLayout()
+        bottom_row_layout.addWidget(self.tile_config_group)
+        bottom_row_layout.addWidget(self.dataset_config_group)
+        main_layout.addLayout(bottom_row_layout)
 
         # Add buttons
         button_box = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
@@ -100,7 +96,6 @@ class Base(QDialog):
         self.tile_config_group.setTitle("Tile Configuration Parameters")
         layout = QFormLayout()
 
-        # Existing tile config code...
         self.tile_size_input = TileSizeInput()
         layout.addRow(self.tile_size_input)
 
@@ -112,9 +107,9 @@ class Base(QDialog):
 
         self.tile_config_group.setLayout(layout)
 
-    def setup_dataset_layout(self):
-        """Set up dataset layout."""
-        self.dataset_group.setTitle("Dataset Parameters")
+    def setup_input_dataset_layout(self):
+        """Set up input dataset layout (source directory)."""
+        self.input_dataset_group.setTitle("Input Dataset Parameters")
         layout = QFormLayout()
 
         # Source Directory
@@ -126,9 +121,12 @@ class Base(QDialog):
         src_layout.addWidget(self.src_button)
         layout.addRow("Source Directory:", src_layout)
 
-        # Name of Destination Dataset
-        self.dst_name_edit = QLineEdit()
-        layout.addRow("Destination Dataset Name:", self.dst_name_edit)
+        self.input_dataset_group.setLayout(layout)
+
+    def setup_output_dataset_layout(self):
+        """Set up output dataset layout (destination directory and name)."""
+        self.output_dataset_group.setTitle("Output Dataset Parameters")
+        layout = QFormLayout()
 
         # Destination Directory
         self.dst_edit = QLineEdit()
@@ -139,36 +137,44 @@ class Base(QDialog):
         dst_layout.addWidget(self.dst_button)
         layout.addRow("Destination Directory:", dst_layout)
 
-        self.dataset_group.setLayout(layout)
+        # Name of Destination Dataset
+        self.dst_name_edit = QLineEdit()
+        layout.addRow("Destination Dataset Name:", self.dst_name_edit)
+
+        self.output_dataset_group.setLayout(layout)
 
     def setup_dataset_config_layout(self):
         """Set up dataset config layout."""
         self.dataset_config_group.setTitle("Dataset Configuration Parameters")
         layout = QFormLayout()
 
-        # Image Extensions
-        ext_layout = QHBoxLayout()
+        # Image Extensions 
+        ext_group = QGroupBox("Image Extensions")
+        ext_form = QFormLayout(ext_group)
+        
         self.input_ext_combo = QComboBox()
         self.input_ext_combo.addItems([".png", ".tif", ".jpeg", ".jpg"])
         self.input_ext_combo.setEditable(True)
+        
         self.output_ext_combo = QComboBox()
         self.output_ext_combo.addItems([".png", ".tif", ".jpeg", ".jpg"])
         self.output_ext_combo.setEditable(True)
-        ext_layout.addWidget(QLabel("Input Ext:"))
-        ext_layout.addWidget(self.input_ext_combo)
-        ext_layout.addWidget(QLabel("Output Ext:"))
-        ext_layout.addWidget(self.output_ext_combo)
-        layout.addRow("Image Extensions:", ext_layout)
+        
+        # Add compression spinbox
+        self.compression_spinbox = QSpinBox()
+        self.compression_spinbox.setRange(0, 100)
+        self.compression_spinbox.setValue(90)
+        self.compression_spinbox.setSingleStep(5)
+        
+        ext_form.addRow("Input Extension:", self.input_ext_combo)
+        ext_form.addRow("Output Extension:", self.output_ext_combo)
+        ext_form.addRow("Compression (0-100):", self.compression_spinbox)
+        
+        layout.addRow(ext_group)
 
-        # Include negative samples
-        self.include_negatives_combo = QComboBox()
-        self.include_negatives_combo.addItems(["True", "False"])
-        self.include_negatives_combo.setEditable(False)
-        self.include_negatives_combo.setCurrentIndex(0)
-        layout.addRow("Include Negative Samples:", self.include_negatives_combo)
-
-        # Train, Validation, and Test Ratios
-        ratios_layout = QHBoxLayout()
+        # Train, Validation, and Test Ratios 
+        ratios_group = QGroupBox("Dataset Split Ratios")
+        ratios_form = QFormLayout(ratios_group)
 
         self.train_ratio_spinbox = QDoubleSpinBox()
         self.train_ratio_spinbox.setRange(0.0, 1.0)
@@ -185,36 +191,56 @@ class Base(QDialog):
         self.test_ratio_spinbox.setSingleStep(0.1)
         self.test_ratio_spinbox.setValue(0.1)
 
-        ratios_layout.addWidget(QLabel("Train:"))
-        ratios_layout.addWidget(self.train_ratio_spinbox)
-        ratios_layout.addWidget(QLabel("Valid:"))
-        ratios_layout.addWidget(self.valid_ratio_spinbox)
-        ratios_layout.addWidget(QLabel("Test:"))
-        ratios_layout.addWidget(self.test_ratio_spinbox)
+        ratios_form.addRow("Train Ratio:", self.train_ratio_spinbox)
+        ratios_form.addRow("Validation Ratio:", self.valid_ratio_spinbox)
+        ratios_form.addRow("Test Ratio:", self.test_ratio_spinbox)
 
-        layout.addRow("Dataset Split Ratios:", ratios_layout)
+        layout.addRow(ratios_group)
 
-        # Number of Visualization Samples
-        self.num_viz_sample_spinbox = QSpinBox()
-        self.num_viz_sample_spinbox.setRange(1, 1000)
-        self.num_viz_sample_spinbox.setValue(25)
-        layout.addRow("# Visualization Samples:", self.num_viz_sample_spinbox)
-
-        # Advanced options (densify factor and smoothing tolerance)
-        advanced_layout = QHBoxLayout()
+        # Advanced options 
+        advanced_group = QGroupBox("Advanced Parameters")
+        advanced_form = QFormLayout(advanced_group)
+        
         self.densify_factor_spinbox = QDoubleSpinBox()
         self.densify_factor_spinbox.setRange(0.0, 1.0)
         self.densify_factor_spinbox.setSingleStep(0.1)
         self.densify_factor_spinbox.setValue(0.5)
+        
         self.smoothing_tolerance_spinbox = QDoubleSpinBox()
         self.smoothing_tolerance_spinbox.setRange(0.0, 1.0)
         self.smoothing_tolerance_spinbox.setSingleStep(0.1)
         self.smoothing_tolerance_spinbox.setValue(0.1)
-        advanced_layout.addWidget(QLabel("Densify:"))
-        advanced_layout.addWidget(self.densify_factor_spinbox)
-        advanced_layout.addWidget(QLabel("Smoothing:"))
-        advanced_layout.addWidget(self.smoothing_tolerance_spinbox)
-        layout.addRow("Advanced Parameters:", advanced_layout)
+        
+        advanced_form.addRow("Densify Factor:", self.densify_factor_spinbox)
+        advanced_form.addRow("Smoothing Tolerance:", self.smoothing_tolerance_spinbox)
+        
+        layout.addRow(advanced_group)
+        
+        # Misc. options
+        misc_group = QGroupBox("Misc.")
+        misc_form = QFormLayout(misc_group)
+        
+        # Include negative samples
+        self.include_negatives_combo = QComboBox()
+        self.include_negatives_combo.addItems(["True", "False"])
+        self.include_negatives_combo.setEditable(False)
+        self.include_negatives_combo.setCurrentIndex(0)
+        misc_form.addRow("Include Negative Samples:", self.include_negatives_combo)
+        
+        # Copy source data
+        self.copy_source_data_combo = QComboBox()
+        self.copy_source_data_combo.addItems(["True", "False"])
+        self.copy_source_data_combo.setEditable(False)
+        self.copy_source_data_combo.setCurrentIndex(1)
+        misc_form.addRow("Copy Source Data:", self.copy_source_data_combo)
+        
+        # Number of Visualization Samples
+        self.num_viz_sample_spinbox = QSpinBox()
+        self.num_viz_sample_spinbox.setRange(1, 1000)
+        self.num_viz_sample_spinbox.setValue(3)
+        misc_form.addRow("# Visualization Samples:", self.num_viz_sample_spinbox)
+        
+        layout.addRow(misc_group)
 
         self.dataset_config_group.setLayout(layout)
 
@@ -365,6 +391,7 @@ class Base(QDialog):
 
         input_ext = self.input_ext_combo.currentText()
         output_ext = self.output_ext_combo.currentText()
+        compression = self.compression_spinbox.value()
         densify_factor = self.densify_factor_spinbox.value()
         smoothing_tolerance = self.smoothing_tolerance_spinbox.value()
         train_ratio = self.train_ratio_spinbox.value()
@@ -372,6 +399,8 @@ class Base(QDialog):
         test_ratio = self.test_ratio_spinbox.value()
         include_negatives = self.include_negatives_combo.currentText()
         include_negatives = True if include_negatives == "True" else False
+        copy_source_data = self.copy_source_data_combo.currentText()
+        copy_source_data = True if copy_source_data == "True" else False
         num_viz_samples = self.num_viz_sample_spinbox.value()
 
         # Perform all validation checks
@@ -424,7 +453,9 @@ class Base(QDialog):
             valid_ratio=valid_ratio,
             test_ratio=test_ratio,
             margins=margins,
-            include_negative_samples=include_negatives
+            include_negative_samples=include_negatives,
+            copy_source_data=copy_source_data,
+            compression=compression,
         )
 
         tiler = YoloTiler(
@@ -432,7 +463,7 @@ class Base(QDialog):
             target=dst,
             config=config,
             progress_callback=progress_callback,
-            num_viz_samples=num_viz_samples,  # Number of samples to visualize
+            num_viz_samples=num_viz_samples,        # Number of samples to visualize
         )
 
         # Copy the class_mapping.json file if it exists
