@@ -3,11 +3,13 @@
 from .TileDataset.QtClassify import Classify as TileClassifyDataset
 from .TileDataset.QtDetect import Detect as TileDetectDataset
 from .TileDataset.QtSegment import Segment as TileSegmentDataset
-from .TileInference.QtBase import Base as TileInference
+from .QtTileCreation import TileCreation
+from .QtTileBatchInference import TileBatchInference
 
 __all__ = [
     'TileClassifyDataset',
     'TileDetectDataset',
     'TileSegmentDataset',
-    'TileInference',
+    'TileCreation',
+    'TileBatchInference'
 ]
\ No newline at end of file
